@@ -4,6 +4,7 @@ from docutils.core import publish_parts
 import yaml
 import os
 
+
 class Course:
     def __init__(self, courseId, lang, title, longDesc, shortDesc, willlearn, requirements, toc, extranalLinks,
                  archived_lessons, active_lessons):
@@ -36,14 +37,14 @@ class Course:
         missing_status = True
         for lesson in self.active_lessons:
             for activity in lesson.active_activies:
-                if activity.activity_type in ['reading', 'quiz']:
-                    if activity.get_src_ext() == 'rst':
+                if activity.type in ['reading', 'quiz']:
+                    if activity.get_src_type() == 'rst':
                         if not Path('_sources/'+lesson.folder+'/'+activity.src).is_file():
                             missing_activities_src.append(
                                 '_sources/'+lesson.folder+'/'+activity.src)
                             missing_activities.append(activity.title)
                             missing_status = False
-                    if activity.get_src_ext() == 'ipynb':
+                    if activity.get_src_type() == 'ipynb':
                         if activity.nbsrc:
                             if not Path(activity.nbsrc).is_file():
                                 missing_activities_src.append(
@@ -56,13 +57,13 @@ class Course:
                                     '_sources/'+lesson.folder+'/'+activity.src)
                                 missing_activities.append(activity.title)
                                 missing_status = False
-                    if activity.get_src_ext() == 'pdf':
+                    if activity.get_src_type() == 'pdf':
                         if not Path('_static/'+activity.src).is_file():
                             missing_activities_src.append(
                                 '_static/'+activity.src)
                             missing_activities.append(activity.title)
                             missing_status = False
-        return {'status': missing_status,'missing_activitie_titles' : missing_activities,'missing_activitie_src' : missing_activities_src}
+        return {'status': missing_status, 'missing_activitie_titles': missing_activities, 'missing_activitie_src': missing_activities_src}
 
     def create_YAML(self, desc):
         self.dict['courseId'] = self.courseId
@@ -83,9 +84,10 @@ class Course:
         for lesson in self.active_lessons:
             tmp_activities = []
             for activity in lesson.active_activies:
-                tmp_activities.append({'type': activity.activity_type,
+                tmp_activities.append({'type': activity.type,
                                        'title': activity.title,
-                                       'file': activity.activity_html_file_src if activity.activity_type in  ['reading','quiz'] and activity.get_src_ext() != 'pdf' else activity.src,
+                                       'src': activity.src,
+                                       'file': activity.html_file_src,
                                        'description': activity.description,
                                        'guid': activity.guid}
                                       )
@@ -98,7 +100,7 @@ class Course:
                                          'description':  lesson.description,
                                          'folder': lesson.folder,
                                          'activities': tmp_activities,
-                                         'archived-activities': tmp_archived,}
+                                         'archived-activities': tmp_archived, }
                                         )
             if len(self.archived_lessons):
                 self.dict['archived-lessons'] = []
@@ -131,7 +133,7 @@ class Course:
             for activity in lesson.active_activies:
                 activity_dict = dict()
                 activity_dict['toc_url'] = activity.toc_url
-                activity_dict['activity_type'] = activity.activity_type
+                activity_dict['type'] = activity.type
                 activity_dict['title'] = activity.title
                 lesson_dict['active_activies'].append(activity_dict)
             course_dict['active_lessons'].append(lesson_dict)
@@ -150,39 +152,36 @@ class Lesson:
 
 
 class Activity:
-    def __init__(self, activity_type, title, src, guid, description, nbsrc):
-        self.activity_type = activity_type
+    def __init__(self, type, title, src, guid, description, nbsrc):
+        self.type = type
         self.title = title
         self.guid = guid
-        self.nbsrc = nbsrc if nbsrc else ''
         self.description = description if description else ''
-        if self.activity_type == 'video':
-            self.src = video_url(src)
-            self.toc_url = self.title.replace(" ", "%20")
-            self.activity_html_file_src =  self.title.replace("?","") + '.html'
-        elif self.activity_type == 'coding-quiz':
-            self.toc_url =  self.title.replace(" ", "%20")
+        self.nbsrc = nbsrc
+        if self.type in ["reading", "quiz", 'coding-quiz']:
             self.src = src
-            self.activity_html_file_src = self.title.replace("?","")  + '.html'
-        # reading, quiz
+        elif self.type == 'video':
+            self.src = yt_id(src)
+        if self.type in ['reading', 'quiz'] and self.get_src_type() == 'rst':
+            self.toc_url = self.src.replace('.rst', '').replace(' ', '%20')
+            self.html_file_src = self.src.replace('.rst', '.html')
         else:
-            self.src = src
-            if self.get_src_ext() == 'ipynb':
-                if self.nbsrc:
-                    self.nbsrc = os.path.join(self.nbsrc[3:], src)
-                self.toc_url =  self.title.replace(" ", "%20")
-                self.activity_html_file_src = self.title.replace("?","")  + '.html'
-            elif self.get_src_ext() == 'pdf':
-                self.toc_url =  self.title.replace(" ", "%20")
-                self.activity_html_file_src = self.title.replace("?","")  + '.html'
-            else:
-                self.toc_url = src.split('.')[0].replace(" ", "%20")
-                self.activity_html_file_src = self.src.replace('.rst','.html').replace("?","") 
-        self.toc_url = normalize(self.toc_url)
-        
-    def get_src_ext(self):
-        if len(self.src.rsplit('.')) > 1:
-            return self.src.rsplit('.')[1]
+            self.html_file_src = normalize(self.title) + '.html'
+            self.rst_file_src = normalize(self.title) + '.rst'
+            self.toc_url = normalize(self.title).replace(" ", "%20")
+        if self.nbsrc and self.get_src_type() == 'ipynb':
+            # root of the nbsrc path is _sources folder, script executes from project root folder
+            # so we need to adjust the path and then add the notebook file
+            self.nbsrc = os.path.join(self.nbsrc[3:], src)
+
+    def get_src_type(self):
+        if self.type in ['reading', 'quiz']:
+            if len(self.src.rsplit('.')) > 1:
+                return self.src.rsplit('.')[1]
+        if self.type == 'vide':
+            return 'video_url'
+        if self.type == 'coding-quiz':
+            return 'coding_quiz_list'
         return ''
 
 
@@ -191,12 +190,15 @@ class ExternalLink:
         self.text = text
         self.link = link
 
+    def __str__(self):
+        return self.text + "<" + self.link + ">"
+
 
 class YamlLoger:
 
     ATR_COURSE_ID = 'courseId'
-    ATR_LANG  = 'lang'
-    ATR_TITLE =  'title'
+    ATR_LANG = 'lang'
+    ATR_TITLE = 'title'
     ATR_DESC = 'description'
     ATR_DESC_LINE = 'description_line'
     ATR_WILL_LEARN = 'willLearn'
@@ -222,7 +224,7 @@ class YamlLoger:
     ATR_LESSON_LINE = '_line'
     ATR_LESSON_TITLE = 'title'
     ATR_LESSON_FOLDER = 'folder'
-    ATR_LESSON_GUID ='guid'
+    ATR_LESSON_GUID = 'guid'
     ATR_LESSON_DESC = 'description'
     ATR_LESSON_NBSRC = 'nbsrc-folder'
     ATR_LESSON_ACTIVITIES = 'lesson_activities'
@@ -244,44 +246,45 @@ class YamlLoger:
     YAML_TYPE_ERROR = 'yaml_type_error'
     ERROR_YAML_LOAD = 'yaml_load_error'
     ERROR_MSGS = {
-                't' : {
-                    ATR_COURSE_ID : 'Missing required attribute "courseId" (Top level).',
-                    ATR_LANG : 'Missing required attribute "lang" (Top level).',
-                    ATR_TITLE : 'Missing required attribute "title" (Top level).',
-                    ATR_DESC : 'Missing required attribute "description" (Top level).',
-                    ATR_WILL_LEARN : 'In "description" (line: {}). Missing required attribute "willLearn".',
-                    ATR_REQUIREMENTS : 'In "description" (line: {}). Missing required attribute "requirements".',
-                    ATR_TOC : 'In "description" (line: {}). Missing required attribute "toc" (Table of content).',
-                    ATR_SHORT_DESC : 'In "description" (line: {}). Missing required attribute "shortDescription".',
-                    ATR_LONG_DESC : 'In "description" (line: {}). Missing required attribute "longDescription".',
-                    ATR_EXTERNAL_LINKS_TEXT : 'In "externalLinks" (line: {}). External link {} is missing required attribute "text".',
-                    ATR_EXTERNAL_LINKS_LINK : 'In "externalLinks" (line: {}). External link {} is missing required attribute "href".',
-                    ATR_LESSONS : 'Missing required attribute "lessons" (Top level).',
-                    ATR_ARCHIVED_LESSON_GUID : 'In "archived-lessons" (line: {}). Lesson {} is missing required attribute "guid".',
-                    DUPLICATE_GUID : 'Duplicated GUID found: {}.',
-                    SOURCE_MISSING : 'Activity "{}" source missing. File should be here:\n>> {}',
-                    YAML_TYPE_ERROR : 'Yaml stucture error.',
-                    ERROR_YAML_LOAD : 'Yaml could not be loaded.'
-                },
-                'l' : {
-                    ATR_LESSON_TITLE : 'In "lessons" (line: {}). Lesson {} is missing the required attribute "title".',
-                    ATR_LESSON_FOLDER : 'In "lessons" (line: {}). Lesson {} is missing the required attribute "folder".',
-                    ATR_LESSON_GUID : 'In "lessons" (line: {}). Lesson {} is missing the required attribute "guid".',
-                    ATR_LESSON_ACTIVITIES : 'In "lessons" (line: {}). Lesson {} is missing the required attribute "activities".',
-                },
-                'a': {
-                    ATR_ACTIVITY_TYPE :'In "lesson" {}. Activity {} (line: {}) is missing the required attribute "type".',
-                    ATR_ACTIVITY_TITLE : 'In "lesson" {}. Activity {} (line: {}) is missing the required attribute "title".',
-                    ATR_ACTIVITY_GUID : 'In "lesson" {}. Activity {} (line: {}) is missing the required attribute "guid".',
-                    ATR_ACTIVITY_SRC : 'In "lesson" {}. Activity {} (line: {}) is missing the source("file" or "url").',
-                    ATR_LESSON_ARCHIVED_ACTIVITIE_GUID : 'In Lesson {} "archived-activities" {} (line: {}) missing required attribute "guid".',
-                    ATR_ACTIVITY_TYPE_VALUE : 'Unsupported activity type {}.',
-                    ATR_ACTIVITY_PROBLEMS : 'In "lesson" {}. Activity {} (line: {}) is missing "problems"',
-                    ATR_FILE : 'In "lesson" {}. Activity {} (line: {}) is missing the source("file" or "url").',
-                    ATR_URL : 'In "lesson" {}. Activity {} (line: {}) is missing the source("file" or "url").',
-                }
+        't': {
+            ATR_COURSE_ID: 'Missing required attribute "courseId" (Top level).',
+            ATR_LANG: 'Missing required attribute "lang" (Top level).',
+            ATR_TITLE: 'Missing required attribute "title" (Top level).',
+            ATR_DESC: 'Missing required attribute "description" (Top level).',
+            ATR_WILL_LEARN: 'In "description" (line: {}). Missing required attribute "willLearn".',
+            ATR_REQUIREMENTS: 'In "description" (line: {}). Missing required attribute "requirements".',
+            ATR_TOC: 'In "description" (line: {}). Missing required attribute "toc" (Table of content).',
+            ATR_SHORT_DESC: 'In "description" (line: {}). Missing required attribute "shortDescription".',
+            ATR_LONG_DESC: 'In "description" (line: {}). Missing required attribute "longDescription".',
+            ATR_EXTERNAL_LINKS_TEXT: 'In "externalLinks" (line: {}). External link {} is missing required attribute "text".',
+            ATR_EXTERNAL_LINKS_LINK: 'In "externalLinks" (line: {}). External link {} is missing required attribute "href".',
+            ATR_LESSONS: 'Missing required attribute "lessons" (Top level).',
+            ATR_ARCHIVED_LESSON_GUID: 'In "archived-lessons" (line: {}). Lesson {} is missing required attribute "guid".',
+            DUPLICATE_GUID: 'Duplicated GUID found: {}.',
+            SOURCE_MISSING: 'Activity "{}" source missing. File should be here:\n>> {}',
+            YAML_TYPE_ERROR: 'Yaml stucture error.',
+            ERROR_YAML_LOAD: 'Yaml could not be loaded.'
+        },
+        'l': {
+            ATR_LESSON_TITLE: 'In "lessons" (line: {}). Lesson {} is missing the required attribute "title".',
+            ATR_LESSON_FOLDER: 'In "lessons" (line: {}). Lesson {} is missing the required attribute "folder".',
+            ATR_LESSON_GUID: 'In "lessons" (line: {}). Lesson {} is missing the required attribute "guid".',
+            ATR_LESSON_ACTIVITIES: 'In "lessons" (line: {}). Lesson {} is missing the required attribute "activities".',
+        },
+        'a': {
+            ATR_ACTIVITY_TYPE: 'In "lesson" {}. Activity {} (line: {}) is missing the required attribute "type".',
+            ATR_ACTIVITY_TITLE: 'In "lesson" {}. Activity {} (line: {}) is missing the required attribute "title".',
+            ATR_ACTIVITY_GUID: 'In "lesson" {}. Activity {} (line: {}) is missing the required attribute "guid".',
+            ATR_ACTIVITY_SRC: 'In "lesson" {}. Activity {} (line: {}) is missing the source("file" or "url").',
+            ATR_LESSON_ARCHIVED_ACTIVITIE_GUID: 'In Lesson {} "archived-activities" {} (line: {}) missing required attribute "guid".',
+            ATR_ACTIVITY_TYPE_VALUE: 'Unsupported activity type {}.',
+            ATR_ACTIVITY_PROBLEMS: 'In "lesson" {}. Activity {} (line: {}) is missing "problems"',
+            ATR_FILE: 'In "lesson" {}. Activity {} (line: {}) is missing the source("file" or "url").',
+            ATR_URL: 'In "lesson" {}. Activity {} (line: {}) is missing the source("file" or "url").',
+        }
     }
 
+
 def duplicates(guid_list):
     seen = {}
     dupes = []
@@ -296,7 +299,7 @@ def duplicates(guid_list):
     return dupes
 
 
-def video_url(src):
+def yt_id(src):
     if len(src) > 11:
         pos = src.find('v=')
         if pos == -1:
@@ -316,6 +319,7 @@ class ActivityTypeValueError(Exception):
         else:
             self.message = None
 
-def normalize(string : str):
-    reserved_chars = ['?','>',':','"','/','\\','|','*']
-    return ''.join(char for char in string if char not in reserved_chars)
\ No newline at end of file
+
+def normalize(string: str):
+    reserved_chars = ['?', '>', ':', '"', '/', '\\', '|', '*']
+    return ''.join(char for char in string if char not in reserved_chars)
